#pragma once

#include <string>
#include <vector>
#include <functional>

typedef std::function< void(std::string& key, std::string& value) > KeyValueCallbackFunction;
typedef std::function< void(std::string& key, std::vector<std::string>& array) > KeyArrayCallbackFunction;

class SonParser
{
public:
	SonParser();
	SonParser(const std::string& source);
	SonParser(const char* source, int& length);

	void SetSource(const std::string& source);
	void SetSource(const char* source, int& length);

	void Parse();
	void Parse(const std::string& source);
	void Parse(const char* source, int& length);

	bool HasParseErrors();
	std::string GetParseError();

	void SetCallbacks(KeyValueCallbackFunction keyValueFunc, KeyArrayCallbackFunction keyArrayFunc);
	void SetKeyValueCallback(KeyValueCallbackFunction func);
	void SetKeyArrayCallback(KeyArrayCallbackFunction func);

private:
	void ResetError();
	void SetError(const char* error, int& position);
	void SetError(std::string& error, int& position);

private:
	const char* source;
	int length;
	bool hasParseErrors;
	std::string parseError;

	KeyValueCallbackFunction keyValueCallback;
	KeyArrayCallbackFunction keyArrayCallback;
};
